import datetime

import cv2
import playsound
from gtts import gTTS
from pushbullet import Pushbullet

import face_recognition

# Teach Face
face_image = face_recognition.load_image_file("YOUR_FACE.jpg")
face_face_encodings = face_recognition.face_encodings(face_image)[0]
known_face_encodings = [face_face_encodings, ]
known_face_names = ["YOUR_NAME", ]

# Pusb Bullet
api_key = "YOUR_ACCESS_TOKEN"
pb = Pushbullet(api_key)

# Recognize Face
video_capture = cv2.VideoCapture(0)
last_time = datetime.datetime.now()


while True:
    # Grab a single frame of video
    ret, frame = video_capture.read()

    # Convert the image from BGR color (which OpenCV uses) to RGB color (which face_recognition uses)
    rgb_frame = frame[:, :, ::-1]

    # Find all the faces and face enqcodings in the frame of video
    face_locations = face_recognition.face_locations(rgb_frame)
    face_encodings = face_recognition.face_encodings(rgb_frame, face_locations)

    # Loop through each face in this frame of video
    for (top, right, bottom, left), face_encoding in zip(face_locations, face_encodings):
        # See if the face is a match for the known face(s)
        matches = face_recognition.compare_faces(known_face_encodings, face_encoding)

        name = "Unknown"

        # If a match was found in known_face_encodings, just use the first one.
        if True in matches:
            first_match_index = matches.index(True)
            name = known_face_names[first_match_index]
        else:
            current_time = datetime.datetime.now()
            if current_time <= last_time + datetime.timedelta(seconds=10):
                tts = gTTS('Intruder Alert ! Intruder Alert.', lang='en')
                tts.save('intruder.mp3')
                playsound.playsound('intruder.mp3')

                # Send Notification
                pb.push_note("Intruder Detected ", "An unknown intruder was detected")
                last_time = current_time

        # Draw a box around the face
        cv2.rectangle(frame, (left, top), (right, bottom), (0, 0, 255), 2)

        # Draw a label with a name below the face
        cv2.rectangle(frame, (left, bottom - 35), (right, bottom), (0, 0, 255), cv2.FILLED)
        font = cv2.FONT_HERSHEY_DUPLEX
        cv2.putText(frame, name, (left + 6, bottom - 6), font, 1.0, (255, 255, 255), 1)

    # Display the resulting image
    cv2.imshow('Video', frame)

    # Hit 'q' on the keyboard to quit!
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break

# Release handle to the webcam
video_capture.release()
cv2.destroyAllWindows()
